@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # 
 # Copyright 2010 Bitly
 #
@@ -34,17 +34,17 @@ def run():
             continue
         try:
             t = Decimal(line)
+            count +=1
+            data[t] = data.get(t, 0) + 1
         except:
-            print >>sys.stderr, "invalid line %r" % line
-        count +=1
-        data[t] = data.get(t, 0) + 1
-    print calc_95(data, count)
+            print("invalid line %r" % line, file=sys.stderr)
+    print(calc_95(data, count))
         
 def calc_95(data, count):
     # find the time it took for x entry, where x is the threshold
     threshold = Decimal(count) * Decimal('.95')
     start = Decimal(0)
-    times = data.keys()
+    times = list(data.keys())
     times.sort()
     for t in times:
         # increment our count by the # of items in this time bucket
@@ -54,6 +54,6 @@ def calc_95(data, count):
 
 if __name__ == "__main__":
     if sys.stdin.isatty() or '--help' in sys.argv or '-h' in sys.argv:
-        print "Usage: cat data | %s" % os.path.basename(sys.argv[0])
+        print("Usage: cat data | %s" % os.path.basename(sys.argv[0]))
         sys.exit(1)
     run()
